@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Bright Computing Holding BV
+# Copyright 2004-2023 Bright Computing Holding BV
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import
+from __future__ import annotations
 
 import logging
 
@@ -24,7 +24,7 @@ from clusterondemandconfig import config
 log = logging.getLogger("cluster-on-demand")
 
 
-class ClusterCommandBase(object):
+class ClusterCommandBase:
     """Base class for all AWS cluster commands.
 
     This class only contains non-public validator methods that are intended to be used by
@@ -33,9 +33,10 @@ class ClusterCommandBase(object):
     check. If the check passes the _validate_xxx methods will simply return control to the
     caller with no return value.
     """
-    def _validate_cluster_names(self):
-        for name in config["name"]:
-            ParamValidator.validate_cluster_name(name)
+    def _validate_cluster_names_len_and_regex(self, names):
+        validate_max_name_len = config["validate_max_cluster_name_length"]
+        for name in names:
+            ParamValidator.validate_cluster_name(name, validate_max_name_len)
 
     def _validate_aws_access_credentials(self):
         if not AWSParamValidator.validate_region(config["aws_region"]):
