@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Bright Computing Holding BV
+# Copyright 2004-2023 Bright Computing Holding BV
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import, division, print_function
+from __future__ import annotations
 
 import collections
 import logging
@@ -22,7 +22,6 @@ from datetime import datetime
 
 import netaddr
 from botocore.exceptions import ClientError
-from six.moves import input, range
 
 import clusterondemand.clustercreate
 from clusterondemand import utils
@@ -50,14 +49,57 @@ from .vpc import create_in_existing_vpc, create_vpc
 
 log = logging.getLogger("cluster-on-demand")
 
+DEFAULT_DISK_SETUP = """
+<diskSetup xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'>
+  <device>
+    <blockdev>/dev/vdb</blockdev>
+    <blockdev>/dev/sda</blockdev>
+    <blockdev mode='cloud'>/dev/sdb</blockdev>
+    <blockdev mode='cloud'>/dev/hdb</blockdev>
+    <blockdev mode='cloud'>/dev/vdb</blockdev>
+    <blockdev mode='cloud'>/dev/xvdb</blockdev>
+    <blockdev mode='cloud'>/dev/xvdf</blockdev>
+    <blockdev mode='cloud'>/dev/nvme1n1</blockdev>
+    <partition id='a2'>
+      <size>max</size>
+      <type>linux</type>
+      <filesystem>xfs</filesystem>
+      <mountPoint>/</mountPoint>
+      <mountOptions>defaults,noatime,nodiratime</mountOptions>
+    </partition>
+  </device>
+</diskSetup>
+"""
+
+DEFAULT_DISK_SETUP_10_AND_ABOVE = """
+<diskSetup xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'>
+  <device>
+    <blockdev>/dev/vdb</blockdev>
+    <blockdev>/dev/sda</blockdev>
+    <blockdev mode='cloud'>/dev/sdb</blockdev>
+    <blockdev mode='cloud'>/dev/hdb</blockdev>
+    <blockdev mode='cloud'>/dev/vdb</blockdev>
+    <blockdev mode='cloud'>/dev/xvdf</blockdev>
+    <partition id='a2'>
+      <size>max</size>
+      <type>linux</type>
+      <filesystem>xfs</filesystem>
+      <mountPoint>/</mountPoint>
+      <mountOptions>defaults,noatime,nodiratime</mountOptions>
+    </partition>
+  </device>
+</diskSetup>
+"""
+
 
 HR = "---------------------------------------------------------------------"
 
 config_ns = ConfigNamespace("aws.cluster.create", "cluster creation parameters")
 config_ns.import_namespace(awscommon_ns)
 config_ns.import_namespace(findimages_ns)
-config_ns.override_imported_parameter("version", default="9.2")
+config_ns.override_imported_parameter("version", default="10.0")
 config_ns.import_namespace(clusterondemand.configuration.clustercreate_ns)
+config_ns.import_namespace(clusterondemand.configuration.clustercreatename_ns)
 config_ns.import_namespace(clusterondemand.configuration.cmd_debug_ns)
 config_ns.override_imported_parameter(
     "head_node_type",
@@ -65,7 +107,7 @@ config_ns.override_imported_parameter(
     help="The instance type must exist in the region you use.")
 config_ns.override_imported_parameter(
     "head_node_root_volume_size",
-    default=25,
+    default=42,
     help="Head node root disk size in GB. Should be bigger than the AMI size.")
 config_ns.add_switch_parameter(
     "validate_parameters",
@@ -75,6 +117,7 @@ config_ns.add_switch_parameter(
         "It can be disabled with --no-validate-parameters (use with care)"
     )
 )
+config_ns.remove_imported_parameter("name")
 config_ns.add_enumeration_parameter(
     "name",
     default=[],
@@ -178,7 +221,7 @@ config_ns.add_enumeration_parameter(
     help=("Tags which are to be configured to for the resources of the cluster "
           "(i.e. --tags tag1=value tag2=value)."),
     parser=lambda str_val: tuple(str_val.split("=")),
-    serializer=lambda val: "{0}={1}".format(*val)
+    serializer=lambda val: "{}={}".format(*val)
 )
 config_ns.add_parameter(
     "clusters",
@@ -187,27 +230,7 @@ config_ns.add_parameter(
 )
 config_ns.add_parameter(
     "node_disk_setup",
-    default="""
-<diskSetup xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'>
-  <device>
-    <blockdev>/dev/vdb</blockdev>
-    <blockdev>/dev/sda</blockdev>
-    <blockdev mode='cloud'>/dev/sdb</blockdev>
-    <blockdev mode='cloud'>/dev/hdb</blockdev>
-    <blockdev mode='cloud'>/dev/vdb</blockdev>
-    <blockdev mode='cloud'>/dev/xvdb</blockdev>
-    <blockdev mode='cloud'>/dev/xvdf</blockdev>
-    <blockdev mode='cloud'>/dev/nvme1n1</blockdev>
-    <partition id='a2'>
-      <size>max</size>
-      <type>linux</type>
-      <filesystem>xfs</filesystem>
-      <mountPoint>/</mountPoint>
-      <mountOptions>defaults,noatime,nodiratime</mountOptions>
-    </partition>
-  </device>
-</diskSetup>
-    """)
+    default=DEFAULT_DISK_SETUP)
 config_ns.add_parameter(
     "aws_availability_zone",
     env="AWS_AVAILABILITY_ZONE",
@@ -351,7 +374,8 @@ config_ns.add_switch_parameter(
     # using a custom validation instead
     validation=[
         requires_other_parameters_to_be_set(["existing_vpc_id"]),
-        _validate_user_confirmed_igw()],  # we want a dedicated error message here
+        _validate_user_confirmed_igw(),  # we want a dedicated error message here
+    ],
     help="By default, when creating the cluster in an existing VPC, the cod-aws tool will "
          "not attempt to configure the Internet access in that VPC by creating and configuring an IGW. "
          "This flag can be used to change this behavior. It may only be used in conjunction with '--existing-vpc-id'. "
@@ -386,6 +410,16 @@ config_ns.add_switch_parameter(
     validation=if_disabled_require_other_paramaters_to_be_disabled(["head_node_assign_public_ip"])
 )
 
+config_ns.add_switch_parameter(
+    "configure_nat_gateway",
+    default=False,
+    advanced=True,
+    help="By default, HA clusters will create and route traffic through a NAT gateway for the private subnet. "
+         "However, non-HA clusters will use their head node as the gateway for the private subnet. Enabling this "
+         "option will require non-HA clusters to also create and route traffic through a NAT gateway for the private"
+         " subnet."
+)
+
 config_ns.add_parameter(
     "head_node_sg_id",
     advanced=True,
@@ -452,14 +486,36 @@ def _validate_head_node_ip_in_cidr(ip_orig: str, cidr: str):
 
 class ClusterCreate(ClusterCommandBase):
 
-    def _validate_params(self):
+    def _validate_params(self, names):
         if config["validate_parameters"]:
-            self._validate_cluster_names()
+            self._validate_cluster_names_len_and_regex(names)
             self._validate_aws_access_credentials()
             self._validate_instance_types()
         else:
             log.warning("Ignoring parameter validation. Use --validate-parameters to enable it.")
 
+    @classmethod
+    def _generate_cluster_names(cls, aws_session, names, cluster_count):
+        if (cluster_count > 1 and len(names) != 1) or not names:
+            raise CODException("You need to either specify one name, one name and --clusters <number>, "
+                               "or a list of names.")
+
+        if cluster_count and cluster_count > 1:
+            base_cluster_name = names[0]
+            names = [f"{base_cluster_name}-{i+1:02}" for i in range(cluster_count)]  # [cluster-01, cluster-02, ...]
+
+        name_counter = collections.Counter(names)
+        duplicate_names = [name for name, name_count in name_counter.items() if name_count > 1]
+        if duplicate_names:
+            raise CODException(f"Duplicate cluster names: {' '.join(duplicate_names)}")
+
+        conflicting_names = {cluster.name for cluster in Cluster.find(aws_session, names)}
+        if conflicting_names:
+            raise UserReportableException(f"Cluster(s) with conflicting name(s) already exists: "
+                                          f"{' '.join(conflicting_names)}")
+
+        return names
+
     def _validate_params_existing_vpc(self, ec2, ec2c):
         # We do NOT want this validation to be gated by 'config["validate_parameters"]'
         # The validation here is too important for this (code is touching existing/production VPCs of the customer).
@@ -549,57 +605,15 @@ class ClusterCreate(ClusterCommandBase):
         return (vpc_name, vpc_id, vpc_cidr_block, all_subnets_texts)
 
     def run(self):
-        self._validate_params()
-
         aws_session = establish_connection_to_aws()
         ec2, ec2c = create_ec2_resource_client(aws_session)
+        names = self._generate_cluster_names(aws_session, config["name"], config["clusters"])
+
+        self._validate_params(names)
 
         if config["existing_vpc_id"]:
             self._validate_params_existing_vpc(ec2, ec2c)
 
-        if config["clusters"] > 1 and len(config["name"]) > 1:
-            raise CODException("You need to either specify one name, one name and --clusters, or a list of names.")
-
-        # TODO messages with --prefix and --count shouldn't be printed to the user, also logic needs improvement
-        names = None
-        prefix = None
-        if config["clusters"] > 1:
-            prefix = config["name"][0]
-        else:
-            names = config["name"]
-
-        count = config["clusters"]
-        if count is not None and count > 1:
-            if prefix is None:
-                raise CODException("You need to specify a name(prefix) to create a number of clusters")
-            if names:
-                raise CODException("You cannot specify both --count and list of names")
-
-            names = ["%s-%02d" % (prefix, i) for i in range(count)]
-        else:
-            if not names:
-                raise CODException("Cluster name not specified. "
-                                   "If you're trying to create only one cluster, add '--name <clustername>'. "
-                                   "If you're trying to create multiple clusters in one go, you need to either "
-                                   "specify --prefix and --count or provide list of several individual names.")
-            # Check for duplicate names
-            cnt = collections.Counter(names)
-            duplicate_names = [name for name, name_count in cnt.items() if name_count > 1]
-            if duplicate_names:
-                raise UserReportableException(
-                    "Duplicate cluster names: %s" % " ".join(duplicate_names)
-                )
-
-            # Add prefix if present
-            if prefix:
-                names = ["%s-%s" % (prefix, name) for name in names]
-
-        conflicting_names = {cluster.name for cluster in Cluster.find(aws_session, names)}
-
-        if conflicting_names:
-            raise UserReportableException("Cluster with this names already exist: %s" %
-                                          " ".join(conflicting_names))
-
         head_node_image = AWSImageSource.pick_head_node_image_using_options(config)
         if not head_node_image.version and config["run_cm_bright_setup"]:
             log.warning(
@@ -648,9 +662,13 @@ class ClusterCreate(ClusterCommandBase):
             number_of_stored_ips = 0
             with open(config["store_head_node_ip"], "w") as f:
                 for cluster in requested:
-                    if cluster.primary_head_node and cluster.primary_head_node.public_ip_address:
+                    if not cluster.primary_head_node:
+                        continue
+                    ip = (cluster.primary_head_node.public_ip_address if config["head_node_assign_public_ip"] else
+                          cluster.primary_head_node.private_ip_address)
+                    if ip:
                         number_of_stored_ips += 1
-                        f.write(cluster.primary_head_node.public_ip_address + " " + cluster.name + "\n")
+                        f.write(f"{ip} {cluster.name}\n")
             log.info("Stored %s IP(s) in %s" %
                      (number_of_stored_ips, config["store_head_node_ip"]))
         if config["store_head_node_id"]:
@@ -666,14 +684,14 @@ class ClusterCreate(ClusterCommandBase):
             if len(cluster_creator.failed) == 1:
                 details = cluster_creator.failed[0].error_message
                 raise UserReportableException(f"Cluster creation failed. Details: {details}")
-            else:
-                details = "\n\n".join([c.error_message for c in cluster_creator.failed])
-                raise UserReportableException(f"Cluster creation failed. \n\nDetails:\n{details}")
-        else:
-            return
+
+            details = "\n\n".join([c.error_message for c in cluster_creator.failed])
+            raise UserReportableException(f"Cluster creation failed. \n\nDetails:\n{details}")
+
+        return
 
 
-class ClusterCreator(object):
+class ClusterCreator:
     Choice = collections.namedtuple("Choice", ["method", "name", "description"])
 
     def __init__(self, requested):
@@ -707,69 +725,66 @@ class ClusterCreator(object):
             ])
 
     def retry(self):
-        Cluster.destroy(self.failed)
-        self.to_create = self.failed
+        utils.multithread_run(lambda cluster: cluster.destroy(), self.failed, config["max_threads"])
 
     def cleanup(self):
-        Cluster.destroy(self.failed)
+        utils.multithread_run(lambda cluster: cluster.destroy(), self.failed, config["max_threads"])
 
     def undo(self):
-        Cluster.destroy(self.requested)
+        utils.multithread_run(lambda cluster: cluster.destroy(), self.requested, config["max_threads"])
 
     def run(self):
         to_create_names = [c.name for c in self.requested]
 
-        self.to_create = self.requested
-
         start = datetime.now()
 
-        while self.to_create:
-            log.debug("Running creation iteration")
-            clusters_to_create = self.to_create
-            self.to_create = None
-            to_setup = []
-            failed_create = []
+        log.debug("Running Cluster creation")
 
-            for cluster in clusters_to_create:
-                try:
-                    # cluster.vpc and cluster.primary_head_node will be written in this call
-                    if config["existing_vpc_id"]:
-                        create_in_existing_vpc(cluster)
-                    else:
-                        create_vpc(cluster)
-                except ClientError as e:
-                    failed_create.append(cluster)
-                    if e.response["Error"]["Code"] == "VpcLimitExceeded":
-                        raise UserReportableException("VPC limit exceeded!")
-                    else:
-                        log.error("VPC Creation failed for cluster %s: %s", cluster.name, str(e))
-                except Exception as e:
-                    log.exception("Create VPC failed for %s: %s", cluster.name, str(e))
-                    cluster.error_message = str(e)
-                    failed_create.append(cluster)
+        to_setup = []
+        failed_create = []
+
+        for cluster in self.requested:
+            try:
+                # cluster.vpc and cluster.primary_head_node will be written in this call
+                if config["existing_vpc_id"]:
+                    create_in_existing_vpc(cluster)
                 else:
-                    to_setup.append(cluster)
+                    create_vpc(cluster)
+            except ClientError as e:
+                failed_create.append(cluster)
+                if e.response["Error"]["Code"] == "VpcLimitExceeded":
+                    cluster.error_message = str(e.response["Error"]["Message"])
+                    raise UserReportableException(str(e.response["Error"]))
 
-            if len(self.requested) > 1 and failed_create:
-                #  if we fail during setup, dont bother running bright-setup
-                self.failed = failed_create
-                self.undo()
-                log.debug("To create left: %s" % len(self.to_create))
-                break
+                cluster.error_message = str(e.response["Error"]["Message"])
+                log.error("VPC Creation failed for cluster %s: %s", cluster.name, str(e))
+            except Exception as e:
+                log.exception("Create VPC failed for %s: %s", cluster.name, str(e))
+                cluster.error_message = str(e)
+                failed_create.append(cluster)
             else:
-                failed_setup = []
-                if to_setup:
-                    failed_setup = start_headnodes(to_setup)
+                to_setup.append(cluster)
 
-                self.failed = failed_create + failed_setup
-                if self.failed:
-                    log.warning("Setup failed for clusters %s",
-                                " ".join([c.name for c in self.failed]))
+        if len(self.requested) > 1 and failed_create:
+            #  if we fail during setup, dont bother running bright-setup
+            log.debug(f"Failed to create clusters {' '.join([cluster.name for cluster in failed_create])}, "
+                      f"undoing all progress")
+            self.failed = failed_create
+            self.undo()
+        else:
+            failed_setup = []
+            if to_setup:
+                failed_setup = start_headnodes(to_setup)
 
-                    choice = self.prompt_for_choice()
+            self.failed = failed_create + failed_setup
+            if self.failed:
+                log.warning("Setup failed for clusters %s",
+                            " ".join([c.name for c in self.failed]))
 
-                    if choice.method:
-                        choice.method()
+                choice = self.prompt_for_choice()
+
+                if choice.method:
+                    choice.method()
 
         failed_names = [c.name for c in self.failed]
 
@@ -855,7 +870,7 @@ def start_headnodes(clusters):
                     log.info("Note that you're creating a cluster in an existing VPC, while at the same time "
                              "not ensuring that IGW exists. There's a risk assigning a public IP might fail.")
 
-                cluster.assign_eips()
+                cluster.attach_eips_to_head_nodes()
             else:
                 utils.cod_log(log,
                               "Will not assign a public IP to the head node "
@@ -864,7 +879,9 @@ def start_headnodes(clusters):
         except ClientError as e:
             failed.append(cluster)
             clusters.remove(cluster)
+            cluster.error_message = str(e.response["Error"]["Message"])
             if e.response["Error"]["Code"] == "AddressLimitExceeded":
+                cluster.error_message = "IP Address limit exceeded!"
                 raise UserReportableException("IP Address limit exceeded!")
         except Exception as e:
             log.warning("Failed to assign IP for cluster %s: %s", cluster.name, e)
@@ -887,8 +904,8 @@ def wait_for_cloud_init(clusters):
     if timeout == 0:
         utils.cod_log(log, "Not waiting for cloud-init to start (--cloud-init-timeout was set to 0)", progress_value)
         return []
-    else:
-        utils.cod_log(log, "Waiting for cloud-init to start (use '--cloud-init-timeout 0' to skip)", progress_value)
+
+    utils.cod_log(log, "Waiting for cloud-init to start (use '--cloud-init-timeout 0' to skip)", progress_value)
 
     wait_until = time.time() + timeout
 
