@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Bright Computing Holding BV
+# Copyright 2004-2023 Bright Computing Holding BV
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,11 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import
+from __future__ import annotations
 
 import logging
 
-from clusterondemand.exceptions import CODException
+from clusterondemand.clusternameprefix import clusterprefix_ns, ensure_cod_prefix
+from clusterondemand.utils import confirm, confirm_ns, log_no_clusters_found, multithread_run
 from clusterondemandaws.base import ClusterCommandBase
 from clusterondemandconfig import ConfigNamespace, config
 
@@ -33,9 +34,12 @@ def run_command():
 
 config_ns = ConfigNamespace("aws.cluster.stop")
 config_ns.import_namespace(awscommon_ns)
+config_ns.import_namespace(clusterprefix_ns)
+config_ns.import_namespace(confirm_ns)
 config_ns.add_repeating_positional_parameter(
-    "name",
-    help="Name(s) of the cluster to stop"
+    "filters",
+    require_value=True,
+    help="Cluster names or patterns. Wildcards are supported (e.g: \\*)",
 )
 config_ns.add_switch_parameter(
     "release_eip",
@@ -49,22 +53,28 @@ config_ns.add_switch_parameter(
 class ClusterStop(ClusterCommandBase):
 
     def _validate_params(self):
-        self._validate_cluster_names()
         self._validate_aws_access_credentials()
 
     def run(self):
         self._validate_params()
 
         aws_session = establish_connection_to_aws()
-        prefix = None
-        names = config["name"]
-        if not names:
-            prefix = config["prefix"]
+        names = [ensure_cod_prefix(name) for name in config["filters"]]
+        clusters = list(Cluster.find(aws_session, names))
 
-        clusters = Cluster.find_some(aws_session, prefix, names)
-        if clusters:
-            log.info("Stopping nodes for clusters %s" % " ".join(c.name for c in clusters))
-            Cluster.stop(clusters, release_eip=config["release_eip"])
+        if not clusters:
+            log_no_clusters_found("stop")
+            return
 
-        else:
-            raise CODException("No clusters found")
+        if not confirm(f"This will stop clusters {' '.join([c.name for c in clusters])} continue?"):
+            return
+
+        log.info(f"Stopping nodes for clusters {' '.join(c.name for c in clusters)}.")
+
+        release_eip = config["release_eip"]
+        if release_eip:
+            log.info("The elastic IP(s) will be released as part of the stop process in order to reduce costs. "
+                     "The next time the cluster is started, it will have a different IP. "
+                     "Use --no-release-eip to retain the same IP(s) in between stop/start operations.")
+
+        multithread_run(lambda cluster: cluster.stop(release_eip), clusters, config["max_threads"])
