@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Bright Computing Holding BV
+# Copyright 2004-2023 Bright Computing Holding BV
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 # limitations under the License.
 
 
-from __future__ import absolute_import, print_function
+from __future__ import annotations
 
 import clusterondemand.configshow
 from clusterondemand.configuration import get_enforcing_config_files, get_system_config_files
