@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Bright Computing Holding BV
+# Copyright 2004-2023 Bright Computing Holding BV
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import, print_function
+from __future__ import annotations
 
 import logging
 
@@ -48,7 +48,7 @@ config_ns.add_enumeration_parameter(
     default=["bcm_version", "created_at"],
     choices=[column[0] for column in AWS_COLUMNS],
     help="Sort results by one (or two) of the columns"
-),
+)
 config_ns.add_enumeration_parameter(
     "columns",
     default=[column[0] for column in AWS_COLUMNS],
