@@ -1,4 +1,4 @@
-# Copyright 2004-2022 Bright Computing Holding BV
+# Copyright 2004-2023 Bright Computing Holding BV
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,32 +12,29 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import
+from __future__ import annotations
 
 import logging
 import re
 import shlex
 import time
-import yaml
-from datetime import datetime
-from typing import Dict, List
 
-import six
 import tenacity
+import yaml
 
 from clusterondemand import utils
 from clusterondemand.bcm_version import BcmVersion
+from clusterondemand.cloudconfig.headcommands import get_ssh_auth_key_commands
 from clusterondemand.clustercreate import enable_cmd_debug_commands
 from clusterondemand.exceptions import CODException, UserReportableException
 from clusterondemand.inbound_traffic_rule import InboundTrafficRule
+from clusterondemandaws.cluster import BCM_TYPE_HEAD_NODE, construct_bright_tags
 from clusterondemandconfig import config
 
 from .brightsetup import generate_bright_setup
 
 log = logging.getLogger("cluster-on-demand")
 
-BCM_TYPE_HEAD_NODE = "Head node"
-
 
 def get_vpc_name_for_cluster(cluster_name):
     return config["fixed_cluster_prefix"] + cluster_name
@@ -127,7 +124,14 @@ def create_vpc(cluster):
 
     log.info("Waiting for head node to be running")
     cluster.primary_head_node.wait_until_running()  # must be running for being usable as gateway
-    _set_default_gateway_subnet(cluster.vpc, private_subnet, cluster.primary_head_node)
+
+    if cluster.is_ha or config["configure_nat_gateway"]:
+        allocation_id = cluster.allocate_address_in_vpc(name="NAT Gateway")["AllocationId"]
+        nat_gateway = cluster.create_nat_gateway(public_subnet, allocation_id=allocation_id)
+        cluster.wait_nat_gateway_state(nat_gateway['NatGatewayId'], 'available')
+        _set_default_gateway_subnet(cluster.vpc, private_subnet, nat_gateway)
+    else:
+        _set_default_gateway_subnet(cluster.vpc, private_subnet, cluster.primary_head_node)
 
     _report_progress(22, "Done setting up VPC for %s" % vpc_name)
 
@@ -148,7 +152,7 @@ def _create_vpc(cluster_name, vpc_name, ec2):
     _report_progress(5, "Creating VPC for %s in %s..." % (vpc_name, config["aws_region"]))
 
     # https://aws.amazon.com/about-aws/whats-new/2020/07/amazon-vpc-resources-support-tag-on-create/
-    tags = _construct_bright_tags(cluster_name, vpc_name)
+    tags = construct_bright_tags(cluster_name, vpc_name)
     kwargs = {"CidrBlock": str(config["vpc_cidr"]),
               "TagSpecifications": [{"ResourceType": "vpc", "Tags": tags}]}
 
@@ -165,7 +169,7 @@ def _create_internet_gateway(vpc, ec2):
     _report_progress(10, "Adding internet connectivity...")
 
     internet_gateway_name = vpc.cluster_name
-    tags = _construct_bright_tags(vpc.cluster_name, internet_gateway_name)
+    tags = construct_bright_tags(vpc.cluster_name, internet_gateway_name)
     kwargs = {"TagSpecifications": [{"ResourceType": "internet-gateway", "Tags": tags}]}
 
     internet_gateway = ec2.create_internet_gateway(**kwargs)
@@ -185,7 +189,7 @@ def _create_public_subnet(vpc, ec2):
         kwargs["AvailabilityZone"] = config["aws_availability_zone"]
 
     public_subnet_name = vpc.cluster_name + " public"
-    tags = _construct_bright_tags(vpc.cluster_name, public_subnet_name, BCM_TYPE_HEAD_NODE)
+    tags = construct_bright_tags(vpc.cluster_name, public_subnet_name, BCM_TYPE_HEAD_NODE)
     kwargs["TagSpecifications"] = [{"ResourceType": "subnet", "Tags": tags}]
 
     public_subnet = vpc.create_subnet(**kwargs)
@@ -204,7 +208,7 @@ def _create_private_subnet(vpc, ec2):
         kwargs["AvailabilityZone"] = config["aws_availability_zone"]
 
     private_subnet_name = vpc.cluster_name + " private"
-    tags = _construct_bright_tags(vpc.cluster_name, private_subnet_name, BCM_TYPE_HEAD_NODE)
+    tags = construct_bright_tags(vpc.cluster_name, private_subnet_name, BCM_TYPE_HEAD_NODE)
     kwargs["TagSpecifications"] = [{"ResourceType": "subnet", "Tags": tags}]
 
     private_subnet = vpc.create_subnet(**kwargs)
@@ -230,9 +234,9 @@ def _set_default_gateway_vpc(vpc, internet_gateway):
     after=tenacity.after_log(log, logging.DEBUG),
     reraise=True
 )
-def _set_default_gateway_subnet(vpc, subnet, internet_gateway):
+def _set_default_gateway_subnet(vpc, subnet, gateway):
     route_table_name = subnet.name + " route table"
-    tags = _construct_bright_tags(vpc.cluster_name, route_table_name)
+    tags = construct_bright_tags(vpc.cluster_name, route_table_name)
     rt_kwargs = {"TagSpecifications": [{"ResourceType": "route-table", "Tags": tags}]}
 
     route_table = vpc.create_route_table(**rt_kwargs)
@@ -242,8 +246,10 @@ def _set_default_gateway_subnet(vpc, subnet, internet_gateway):
     route_table.associate_with_subnet(SubnetId=subnet.id)
 
     kwargs = {"DestinationCidrBlock": "0.0.0.0/0"}
-    if "ec2.Instance" == internet_gateway.__class__.__name__:
-        kwargs["InstanceId"] = internet_gateway.id
+    if "ec2.Instance" == gateway.__class__.__name__:
+        kwargs["InstanceId"] = gateway.id
+    elif "NatGatewayId" in gateway:
+        kwargs["NatGatewayId"] = gateway['NatGatewayId']
     route_table.create_route(**kwargs)
 
 
@@ -275,9 +281,18 @@ def _create_head_node(vpc, hostname, public_subnet, head_node_image, head_node_s
         echo '%s' > /root/cm/cm-bright-setup.conf
     """ % yaml.dump(bright_setup, default_flow_style=False).replace("'", '"')
 
+    from .clustercreate import DEFAULT_DISK_SETUP, DEFAULT_DISK_SETUP_10_AND_ABOVE  # avoid circular import
+
+    if config["node_disk_setup"] is not DEFAULT_DISK_SETUP:  # user-supplied disk setup
+        disk_setup = config["node_disk_setup"]
+    elif head_node_image.version and BcmVersion(head_node_image.version) >= BcmVersion("10.0"):
+        disk_setup = DEFAULT_DISK_SETUP_10_AND_ABOVE
+    else:
+        disk_setup = DEFAULT_DISK_SETUP
+
     cloud_init_script += """
         echo "%s" > /root/cm/node-disk-setup.xml
-    """ % config["node_disk_setup"]
+    """ % disk_setup
 
     # The code below the code that is getting patched here still support only 1 or 2 subnets. And that's fine.
     # At this point we simply want to enable creating a cluster in a env with more than 2 subnets (while allowing the
@@ -308,14 +323,13 @@ def _create_head_node(vpc, hostname, public_subnet, head_node_image, head_node_s
     if config["ssh_key_pair"]:
         log.info("Key pair specified")
         kwargs["KeyName"] = config["ssh_key_pair"]
-    if config["ssh_pub_key_path"]:
+
+    auth_key_commands = get_ssh_auth_key_commands()
+    if auth_key_commands:
         log.info("Public key specified")
-        with open(config["ssh_pub_key_path"]) as pub_key:
-            cloud_init_script += """
-            mkdir -p /root/.ssh/
-            echo '# COD injected public key' >> /root/.ssh/authorized_keys
-            echo '%s' >> /root/.ssh/authorized_keys
-            """ % pub_key.read()
+        cloud_init_script += "mkdir -p /root/.ssh/\n"
+        cloud_init_script += "\n".join(auth_key_commands)
+        cloud_init_script += "\n"
 
     cloud_init_script += """
         echo %s | chpasswd
@@ -364,8 +378,8 @@ def _create_head_node(vpc, hostname, public_subnet, head_node_image, head_node_s
 
     head_node_instance_and_ebs_name = f"{vpc.cluster_name} (Bright COD-AWS {config['version']} head node)"
 
-    headnode_tags = _construct_bright_tags(vpc.cluster_name, head_node_instance_and_ebs_name, BCM_TYPE_HEAD_NODE)
-    headnode_ebs_tags = _construct_bright_tags(vpc.cluster_name, head_node_instance_and_ebs_name)
+    headnode_tags = construct_bright_tags(vpc.cluster_name, head_node_instance_and_ebs_name, BCM_TYPE_HEAD_NODE)
+    headnode_ebs_tags = construct_bright_tags(vpc.cluster_name, head_node_instance_and_ebs_name)
 
     kwargs["TagSpecifications"] = [
         {"ResourceType": "instance", "Tags": headnode_tags},
@@ -473,25 +487,7 @@ def _find_main_route_table(vpc):
     return route_tables[0]
 
 
-def _construct_bright_tags(cluster_name, obj_name, bcm_type=None):
-    tags = {
-        "BCM Created at": datetime.utcnow().isoformat() + "Z",  # fix missing timezone
-        "BCM Created by": utils.get_user_at_fqdn_hostname(),
-        "BCM Cluster": cluster_name,
-        "BCM Bursting": "on-demand",
-        # This one has to follow a slightly different naming convention (i.e. no "BCM"
-        # prefix), because this tag has a special meaning for AWS.
-        "Name": obj_name
-    }
-    if bcm_type:
-        tags["BCM Type"] = bcm_type
-
-    tags.update(dict((str(k), str(v)) for k, v in config["cluster_tags"]))
-    tags = [{"Key": key, "Value": value} for key, value in six.iteritems(tags)]
-    return tags
-
-
-def get_aws_tag(tags: List[Dict[str, str]], name: str, default: str = None) -> str:
+def get_aws_tag(tags: list[dict[str, str]], name: str, default: str | None = None) -> str:
     """
     For some mysterious reason, boto3 stores tags as a list of dictionaries:
     [{'Key': 'Name', 'Value': 'my-value-foo'}].
@@ -505,5 +501,5 @@ def get_aws_tag(tags: List[Dict[str, str]], name: str, default: str = None) -> s
 
     if default is not None:
         return default
-    else:
-        raise CODException(f"The AWS tag '{name}' was not found within {len(tags)} tags.")
+
+    raise CODException(f"The AWS tag '{name}' was not found within {len(tags)} tags.")
